import java.awt.Point;
import java.util.ArrayList;


public class CollisionSet {
	/* This class represents a set of collisions between two Weights
	 * Duplicate Collisions are not added
	 * Collisions are removed if the Weights are no longer touching
	 * or Both Weights have stopped moving
	 * This class handles the redirecting of the Weights that collide using the
	 * handleCollision() method
	 */
	private ArrayList<Collision> collisions;
	
	public CollisionSet() {
		collisions = new ArrayList<Collision>();
	}
	
	public void add(Collision c) {
		if (!includes(c)) {
			// Don't handle same collision twice
			collisions.add(c);
			handleCollision(c);
		}		
	}
	public boolean isTouching(Weight w1, Weight w2) {
		// Answer whether two Weights are touching
		Point p = w1.getLocation();
		int distance = (p.x - w2.getLocation().x) * (p.x - w2.getLocation().x) +
					   (p.y - w2.getLocation().y) * (p.y - w2.getLocation().y);
		if ((distance <= 4 * (Weight.RADIUS * Weight.RADIUS)) && (w2 != w1))
			return true;
		return false;
	}
	
	public void add(Weight s1, Weight s2) {
		if (!includes(s1,s2)) add(new Collision(s1,s2) );
	}

	public boolean includes(Collision c) {
		//answer if set already contains collision c, false otherwise
		for(Collision collision: collisions) 
			if (c == collision) return true;  
		return false; 
	}

	public boolean includes(Weight s1, Weight s2) {
		//answer whether set currently has a collision between s1 and s2	     
		for(Collision collision: collisions){
			if( collision.includes(s1) && collision.includes(s2) ) return true; 
		} 
		return false; 
	}
	   
	public void removeOldCollisions() {
		//remove collisions if the Weights are no longer touching or
		//both Weights have stopped moving	     
		Collision old = null;
		for(Collision collision: collisions){
	        if (!isTouching(collision.Weight1, collision.Weight2)) 
	        	old = collision;
	        if ((!collision.Weight1.isMoving()) && (!collision.Weight2.isMoving())) 
	        	old = collision; 
		} 
		if (old != null ) {
			collisions.remove(old);
	     }
	}

	public void handleCollision(Collision c) {
		// set the new velocities of the Weights in the collision c
		// Here are simplifying assumptions:
		// Weight 1 moving and strikes Weight2
		// disregard initial motion of Weight 2 --its probably stopped
		// if Weight 1 is going up, both will have upwards final velocity
		// if Weight 1 is going down, both should have downward velocity
		// if Weight 1 is going left it will bounce right & vice versa
		// Using these simplifications we do all the math in one quadrant
		// using absolute values and use Weight 1's initial directions to
		// set the directions for the final velocities

	 
		Weight Weight1 = c.Weight1;   //assume c.Weight1 is the moving Weight
		Weight Weight2 = c.Weight2;

	         
		if(!Weight1.isMoving() ) return; // Weight1 must be moving

		double v = Weight1.velocity(); 	// velocity vector magnitude

		double dx = (double) (Weight2.getLocation().x - Weight1.getLocation().x);
		double dy = (double) (Weight1.getLocation().y - Weight2.getLocation().y);
		dx = Math.abs(dx); 		//horizontal distance between Weights
		dy = Math.abs(dy); 		//vertical distance between Weights
	         
		double dist12 = Math.sqrt (dx*dx + dy*dy); //distance between Weights
		if (dist12 == 0.0) return; 	//can do divide by zero

		//determine angle of line of impact with horizontal
		double angle_b = Math.asin(dy/dist12);

		//determine angle of Weight 1 velocity with vertical 
		double angle_d = Math.asin(Math.abs(Weight1.getvx())/v);

		//determine angle of Weight 1 velocity line of impact
		double angle_a = (3.14159/2.0) - angle_b - angle_d;

		//determine angle of Weight 1 departure with horizontal
		double angle_c = angle_b - angle_a;

		double v1, v2; //new velocity vectors;
		v1 = v * Math.abs(Math.sin(angle_a));
		v2 = v* Math.abs(Math.cos(angle_a));

		double v1x,v1y,v2x,v2y;
	       
		v1x = v1 * Math.abs(Math.cos(angle_c));
		v1y = v1 * Math.abs(Math.sin(angle_c));
		v2x = v2 * Math.abs(Math.cos(angle_b));
		v2y = v2 * Math.abs(Math.sin(angle_b));

		//set directions based on initial direction of hitting Weight
		//set horizontal directions
		if(Weight1.getvx() > 0) {		// Weight1 is going right
			if(Weight1.getLocation().x < Weight2.getLocation().x) 
				v1x = -v1x;
			else v2x = -v2x;
		}  
		else {
			if(Weight1.getLocation().x > Weight2.getLocation().x)
				v2x = -v2x;
			else v1x = -v1x;
		}
		//set vertical directions
		if(Weight1.getvy() > 0) {		// Weight1 is going right
			if(Weight1.getLocation().y < Weight2.getLocation().y) 
				v1y = -v1y;
			else v2y = -v2y;
		}  
		else {
			if(Weight1.getLocation().y > Weight2.getLocation().y)
				v2y = -v2y;
			else v1y = -v1y;
		}	                 
	         //if(Weight1.getvy() < 0) {v1y = -v1y; v2y = -v2y;} 
	        
		Weight1.setvx(v1x); //set new velocities for Weights
		Weight1.setvy(v1y);
		Weight2.setvx(v2x);
		Weight2.setvy(v2y);
		
	}
	
}
